
;**************************************************************************
;*                    Blikacka s PCA9500                                  *
;**************************************************************************
;**************************************************************************
;*                                                                        *
;*          v 1.12 - 28.04.2017                                           *
;*                                                                        *
;**************************************************************************
;*          Hardw.: PIC12F1822 used                                       *
;*          OSC.......: Internal osc. used     POWER.....:   3,3V ext.    *
;**************************************************************************


;Vss (Pin 8) -- GND
;RA0 (Pin 7) -- not used
;RA1 (Pin 6) -- I2C SCL
;RA2 (Pin 5) -- I2C SDA
;RA3 (Pin 4) -- I2C INT
;RA4 (Pin 3) -- not used
;RA5 (Pin 2) -- Green LED
;Vdd (Pin 1) -- +3,3V

;--------------------------------------------------------------------------
;--------------------------------------------------------------------------
;data format for PIC EEPROM
;EEPROM Address 00h:
;	AAh = overwrite I2C EEPROM with the PIC EEPROM (if the context of I2C EEPROM is different than PIC EEPROM)
;	FFh = use I2C EEPROM
;	anything else = use test03 routine

;EEPROM Address 01h: data length
;EEPROM Address 02-03h: not used

;EEPROM Addresses 04h - FFh: data for output
;--------------------------------------------------------------------------
;--------------------------------------------------------------------------
;data format for I2C EEPROM
;EEPROM Address 00h:
;	01 = use I2C EEPROM
;	FF = use test03 routine
;	anthing else = use test03 routine

;EEPROM Addresses 01h: data length

;EEPROM Addresses 02h and 02h: not used
;EEPROM Addresses 04h - FFh: data for output
;--------------------------------------------------------------------------
;--------------------------------------------------------------------------

 INCLUDE "P12F1822.INC"


_U001			     EQU     B'00100110000100'		;see Page 34 of Datasheet
_U002			     EQU     B'01100011111111'		;see Page 35 of Datasheet


	__CONFIG	_CONFIG1, _U001
	__CONFIG	_CONFIG2, _U002


;==========================================================================
;       Variable Definition
;==========================================================================
TIMER1		EQU	H'20'		;delay routine
TIMER2		EQU	H'21'		; "	"	"
TIMER3		EQU	H'22'		;delay routine

v_ee_add	EQU	H'23'
v_ee_byte	EQU	H'24'
v_byte		EQU	H'25'
v_count		EQU	H'26'
v_byte_pic	EQU	H'27'
v_byte_i2c	EQU	H'28'
v_signal	EQU	H'29'
;--------------------------------------------------------------------------
;--------------------------------------------------------------------------


;==========================================================================

		ORG	0		;Reset vector address
		goto	start

;----------------------------------------------------------------------------
;----------------------------------------------------------------------------
;
;	       **********************************
;              **  RESET :  main boot routine  **
;              **********************************

start		nop
;
		clrf	INTCON		;disable interrupts

		movlw	B'00000000'
		movwf	OPTION_REG	;see datasheet Page 151

		movlw	H'03'
		movwf	BSR		;select BANK 3
		clrf	ANSELA

		movlw	H'01'
		movwf	BSR		;select BANK 1
		movlw	B'11011111'
		movwf	TRISA

		movf	OSCCON,0
		andlw	B'11101111'
		iorlw	B'01101000'
		movwf	OSCCON		;set 4MHz internal clock (default after reset is 500kHz)

		movlw	H'04'
		movwf	BSR		;select BANK 4
		movlw	B'00111111'
		movwf	WPUA		;enable weak pull-ups

		clrf	BSR		;select BANK 0

		clrf	PORTA

;----------------------------------------------------------------------------
;----------------------------------------------------------------------------
		movlw	H'FF'
		movwf	PORTA
		call	d2

		call	i2c_init

;----------------------------------------------------------------------------
;----------------------------------------------------------------------------
;----------------------------------------------------------------------------
;----------------------------------------------------------------------------
; 1. check the PIC EEPROM
;----------------------------------------------------------------------------
;----------------------------------------------------------------------------
start_up1	clrw
		movwf	v_ee_add
		call	ee_pic_read
		xorlw	H'AA'
		btfsc	STATUS,Z
		call	i2c_ee_rewrite
;----------------------------------------------------------------------------
;----------------------------------------------------------------------------
; 2. check the I2C EEPROM
;----------------------------------------------------------------------------
;----------------------------------------------------------------------------
start_up2	clrw
		movwf	v_ee_add
		call	ee_i2c_read
		xorlw	H'01'
		btfsc	STATUS,Z
		goto	i2c_ee_demo
		goto	test03
;----------------------------------------------------------------------------
;----------------------------------------------------------------------------
; 3. rewrite the I2C EEPROM, if there is a difference to PIC EEPROM
;----------------------------------------------------------------------------
;----------------------------------------------------------------------------
i2c_ee_rewrite	nop
;first to compare the content of the data in EEPROMs
		movlw	H'01'
		movwf	v_ee_add
		call	ee_pic_read
		movwf	v_count

		movlw	H'04'
		movwf	v_ee_add

		movlw	B'11111010'
		movwf	v_byte
		call	i2c_port_send

cmp_loop	call	ee_pic_read
		movwf	v_byte_pic
		call	ee_i2c_read
		movwf	v_byte_i2c

		movf	v_byte_pic,0
		xorwf	v_byte_i2c,0
		btfss	STATUS,Z
		goto	i2c_ee_rewr2		;we have found a difference

		incf	v_ee_add,1
		decf	v_count,1
		movf	v_count,0
		btfss	STATUS,Z
		goto	cmp_loop
		return
;
;overwrite the whole I2C EEPROM from PIC EEPROM
i2c_ee_rewr2	clrw
		movwf	v_ee_add
		movlw	H'01'
		movwf	v_ee_byte
		call	ee_i2c_write
		call	dr2

;data length
		movlw	H'01'
		movwf	v_ee_add
		call	ee_pic_read
		movwf	v_ee_byte
		movwf	v_count			;bytes to copy
		call	ee_i2c_write
		call	dr2

;write data from PIC do I2C EEPROM
		movlw	H'04'
		movwf	v_ee_add

		movlw	B'01011111'
		movwf	v_signal

write_loop	call	ee_pic_read
		call	ee_i2c_write
		incf	v_ee_add,1

		movf	v_signal,0
		movwf	v_byte
		call	i2c_port_send
		call	d2
		movf	v_signal,0
		xorlw	B'11111111'
		iorlw	B'00001111'
		movwf	v_signal

		decf	v_count,1
		movf	v_count,0
		btfss	STATUS,Z
		goto	write_loop

		movlw	H'FF'
		call	i2c_port_send
;change the flag in PIC EEPROM - data has been written
;....
		return
;----------------------------------------------------------------------------
;----------------------------------------------------------------------------
; 4. run the demo based on I2C EEPROM data
;----------------------------------------------------------------------------
;----------------------------------------------------------------------------
i2c_ee_demo	movlw	D'001'
		movwf	v_ee_add
		call	ee_i2c_read
		movwf	v_count

		movlw	D'004'
		movwf	v_ee_add
i2c_ee_d_loop	call	ee_i2c_read
		movwf	v_byte
		call	i2c_port_send
		call	d2
		incf	v_ee_add,1
		decf	v_count,1
		movf	v_count,0
		btfss	STATUS,Z
		goto	i2c_ee_d_loop
		goto	i2c_ee_demo
;--------------------------------------------------------------------
;--------------------------------------------------------------------
;XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX
;XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX
;--------------------------------------------------------------------
;--------------------------------------------------------------------
i2c_port_send	call	i2c_start
		movlw	B'01000000'
		call	i2c_send
		movf	v_byte,0
		call	i2c_send
		call	i2c_stop
		return
;--------------------------------------------------------------------
;--------------------------------------------------------------------
;XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX
;XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX
;--------------------------------------------------------------------
;--------------------------------------------------------------------
;I2C EEPROM read
;   address is stored in v_ee_add
;   result will be stored in "v_ee_byte" and in W
;
ee_i2c_read	call	i2c_start
		movlw	B'10100000'
		call	i2c_send
		movf	v_ee_add,0
		call	i2c_send
		call	i2c_restart
		movlw	B'10100001'
		call	i2c_send
		call	i2c_receive
		movwf	v_ee_byte
		call	i2c_not_ack
		call	i2c_stop
		movf	v_ee_byte,0
		return
;--------------------------------------------------------------------
;--------------------------------------------------------------------
;I2C EEPROM write - write 1 byte to EEPROM
;   address is stored in v_ee_add
;   byte to be stored is expected in v_ee_byte
;
ee_i2c_write	call	i2c_start
		movlw	B'10100000'
		call	i2c_send
		movf	v_ee_add,0
		call	i2c_send
		movf	v_ee_byte,0
		call	i2c_send
		call	i2c_stop
		return
;--------------------------------------------------------------------
;--------------------------------------------------------------------
;PIC EEPROM read
;   address is stored in v_ee_add
;   result will be stored in "v_ee_byte" and W register
;
ee_pic_read	movf	v_ee_add,0
		banksel	EEADRL
		movwf	EEADRL
;Address to read
		bcf	EECON1, CFGS
		bcf	EECON1, EEPGD
		bsf	EECON1, RD
		movf	EEDATL,0
		clrf	BSR
		movwf	v_ee_byte
		return
;--------------------------------------------------------------------
;--------------------------------------------------------------------


;----------------------------------------------------------------------------
;----------------------------------------------------------------------------
; TEST03 Demo
;----------------------------------------------------------------------------
;----------------------------------------------------------------------------
test03		call	i2c_start
		movlw	B'01000000'
		call	i2c_send


test03l		movlw	B'11110110'
		call	i2c_send
		call	d2
		movlw	B'11111001'
		call	i2c_send
		call	d2
		movlw	B'10011111'
		call	i2c_send
		call	d2
		movlw	B'01101111'
		call	i2c_send
		call	d2
		movlw	B'10011111'
		call	i2c_send
		call	d2
		movlw	B'11111001'
		call	i2c_send
		call	d2
		goto	test03l
;--------------------------------------------------------------------
;--------------------------------------------------------------------

;XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX
;---------------------------------------------------------------------------------------
;---------------------------------------------------------------------------------------
;XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX

;		*********************************************
;		*               delay routine               *
;		*********************************************

;------------------------------------

d2		movlw	D'255'
		movwf	TIMER2
DEL_LOOP1	movlw	D'255'
		movwf	TIMER1

DEL_LOOP2	decfsz	TIMER1,F
		goto	DEL_LOOP2
		decfsz	TIMER2,F
		goto	DEL_LOOP1
		return
;------------------------------------
dr2		call	d2
		call	d2
		return
;------------------------------------
;------------------------------------
;------------------------------------
 include "90_VeryEasyMaster_I2C_comm_3.inc"

;------------------------------------
;------------------------------------

;XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX
;XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX

		END